/* x11test.c - this file is part of $PROJECT_NAME_HERE$
 *
 * Copyright (C) 2006 Wolfgang Sourdeau
 *
 * Author: Wolfgang Sourdeau <Wolfgang@Contre.COM>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xproto.h>

static int (*PreviousErrorHandler) (Display *xDisplay, XErrorEvent *error);

static void PrepareGrabs (Display *xDisplay)
{
  Window rootWindow;
  KeySym sym;
  KeyCode code;

  rootWindow = DefaultRootWindow(xDisplay);

  sym = XStringToKeysym ("Print");
  if (sym != NoSymbol)
    {
      code = XKeysymToKeycode (xDisplay, sym);
      if (code)
        {
          XSelectInput (xDisplay, rootWindow, KeyPressMask | KeyReleaseMask);
          XGrabKey (xDisplay, code, AnyModifier, rootWindow, 1, GrabModeAsync,
                    GrabModeAsync);
        }
    }
}

static int ErrorHandler (Display *xDisplay, XErrorEvent *error)
{
  int rc;

  if (error->request_code == X_GrabKey)
    {
      fprintf (stderr, "failure grabbing key\n");
      rc = 0;
    }
  else
    rc = PreviousErrorHandler (xDisplay, error);

  return 0;
}

int main()
{
  Display *xDisplay;
  XEvent event;
  
  xDisplay = XOpenDisplay (NULL);
  if (xDisplay)
    {
      PreviousErrorHandler = XSetErrorHandler (ErrorHandler);
      PrepareGrabs (xDisplay);
      while (1)
        {
          while (!XCheckMaskEvent (xDisplay, KeyPressMask, &event))
            {
              printf ("no event...\n");
              usleep (150000);
            }
          printf ("%s\n", XKeysymToString (XKeycodeToKeysym (xDisplay,
  event.xkey.keycode, 0)));
        }
    }
  else
    fprintf (stderr, "error opening display");

  return 0;
}
